require 'spec_helper'

describe Paged do
  context 'a list of items' do
    before :each do
      @list = ['apple', 'banana', 'cat', 'dog', 'elephant', 'frog', 'goat', 'house', 'ice', 'john']
      @paged_list = Paged.new @list, :page_size => 3
    end
    
    it 'should show first page of items' do
      @paged_list.should =~ ['apple', 'banana', 'cat']
      @paged_list.next_page.should == 2
      @paged_list.previous_page.should_not be
    end
    
    it 'should show second page of items' do
      @paged_list.page = 2
      @paged_list.should =~ ['dog', 'elephant', 'frog']
      @paged_list.next_page.should == 3
      @paged_list.previous_page.should == 1
    end
    
    it 'should show incomplete last page' do
      @paged_list.page = 4
      @paged_list.should =~ ['john']
      @paged_list.next_page.should_not be
      @paged_list.previous_page.should == 3
    end
    
    it 'should show nothing after last page' do
      @paged_list.page = 99
      @paged_list.should have(:no).items
      @paged_list.next_page.should_not be
      @paged_list.previous_page.should_not be
    end
    
    it 'should show nothing before first page' do
      @paged_list.page = -1
      @paged_list.should have(:no).items
      @paged_list.next_page.should_not be
      @paged_list.previous_page.should_not be
    end
    
    it 'should raise error for nil or non positive page size' do
      expect { Paged.new @list, :page_size => nil }.to raise_error ArgumentError
      expect { Paged.new @list, :page_size => 0 }.to raise_error ArgumentError
      expect { Paged.new @list, :page_size => -1 }.to raise_error ArgumentError
    end
    
    it 'should raise error for nil or non number page' do
      expect { @paged_list.page = nil }.to raise_error ArgumentError
      expect { @paged_list.page = "2" }.to raise_error ArgumentError
    end
    
    it 'should show one page with no previous or next when page size matches list size' do
      paged_list = Paged.new ['apple', 'banana'], :page_size => 2
      paged_list.should =~ ['apple', 'banana']
      paged_list.next_page.should_not be
      paged_list.previous_page.should_not be
    end
    
    it 'should show two pages when list size is one greater then page size' do
      paged_list = Paged.new ['apple', 'banana', 'cake'], :page_size => 2
      paged_list.should =~ ['apple', 'banana']
      paged_list.next_page.should == 2
      paged_list.previous_page.should_not be
    end
    
    it 'should iterate page from list' do
      x = 0
      @paged_list.each { |item| x += 1 }
      x.should == 3
    end
  end
end